/*
 * spi.h - Serial Peripheral Interface (master mode driver)
 *

    Copyright (c) 2010 Stingworks
    Copyright (c) 2007 Stefan Engelke <mbox@stefanengelke.de>

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation 
    files (the "Software"), to deal in the Software without 
    restriction, including without limitation the rights to use, copy, 
    modify, merge, publish, distribute, sublicense, and/or sell copies 
    of the Software, and to permit persons to whom the Software is 
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be 
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
    DEALINGS IN THE SOFTWARE.

 * File History:
 *  
 *   12 FEB 10: imported from Stefan Engelke's spi.h and spi.c
 *
 */


#ifndef _SPI_H_
#define _SPI_H_

#include <avr/io.h>


void spi_init()
// Initialize pins for spi communication
{
	// note: for 1MHz uC clock, SPI2X,SPR1,SPR0 = 1,0,1 works great!
    
    SPCR = ((1<<SPE)|               // SPI Enable
            (0<<SPIE)|              // SPI Interupt Enable
            (0<<DORD)|              // Data Order (0:MSB first / 1:LSB first)
            (1<<MSTR)|              // Master/Slave select   
            (1<<SPR1)|(1<<SPR0)|    // SPI Clock Rate
            (0<<CPOL)|              // Clock Polarity 
	    			    //  (0:SCK low / 1:SCK hi when idle)
            (0<<CPHA));             // Clock Phase 
    				    //  (0:leading / 1:trailing edge sampling)

    //SPSR = (1<<SPI2X);              // Double Clock Rate

    // set data direction (correct for atmega168)
    DDRB |= 1<<PB5; //SCK
    DDRB |= 1<<PB3; //MOSI
    DDRB &= ~(1<<PB4); //MISO

}

void spi_transfer (uint8_t * dataout, uint8_t * datain, uint8_t len)
// Shift full array through target device
{
       uint8_t i;      
       for (i = 0; i < len; i++) {
             SPDR = dataout[i];
             while((SPSR & (1<<SPIF))==0);
             datain[i] = SPDR;
       }


       PORTD |= 1<<3 | 1<<4 | 1<<5;
}

void spi_read (uint8_t * datain, uint8_t len)
// Shift full array from target device but write only zeros 
{
       uint8_t i;      
       for (i = 0; i < len; i++) {
             SPDR = 0;
             while((SPSR & (1<<SPIF))==0);
             datain[i] = SPDR;
       }

       PORTD |= 1<<3 | 1<<4 | 1<<5;
}

void spi_write (uint8_t * dataout, uint8_t len)
// Shift full array to target device without receiving any byte
{
       uint8_t i;      
       for (i = 0; i < len; i++) {
             SPDR = dataout[i];
             while((SPSR & (1<<SPIF))==0);
       }

       PORTD |= 1<<3 | 1<<4 | 1<<5;
}

uint8_t spi_transfer_byte (uint8_t data)
// Clocks only one byte to target device and returns the received one
{
    SPDR = data;
    while((SPSR & (1<<SPIF))==0);
    return SPDR;

    PORTD |= 1<<3 | 1<<4 | 1<<5;
}


#endif /* _SPI_H_ */

