/*
 * usart.h - USART driver
 *
 *  Copyright (c) 2010 Stingworks
 *
 
    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation 
    files (the "Software"), to deal in the Software without 
    restriction, including without limitation the rights to use, copy, 
    modify, merge, publish, distribute, sublicense, and/or sell copies 
    of the Software, and to permit persons to whom the Software is 
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be 
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
    DEALINGS IN THE SOFTWARE.

 *
 *
 * File History:
 *  
 *  10 FEB 2010:  rewritten using several examples
 *
 */


void usart_init(uint16_t ubrr_value) {

	//set baud rate
	UBRR0L = (uint8_t) ubrr_value;
	UBRR0H = (uint8_t) (ubrr_value>>8);

	//enable reciever and transmitter
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);

	//Set format: Asynch, 8N1:
	UCSR0C = (0<<USBS0)|(3<<UCSZ00);
}

uint8_t usart_read_ready(void) {
	return (UCSR0A & (1<<RXC0)) != 0;
}

uint8_t usart_getc(void) {

	// block until a charicter is ready
	while (!usart_read_ready());

	// now return recvd char
	return UDR0;
}

void usart_double_speed(void) {
	UCSR0A |= 1<<U2X0;
}

void usart_putc(uint8_t b) {
	
	//block until transmitter is ready
	while(!(UCSR0A & (1<<UDRE0))); 

	//write
	UDR0 = b;
}

