/* Name: hidstream.c
 * Project: hid-stream
 * Author: Stingworks
 * Creation Date: 2010-07-18
 * Tabsize: 4
 * Copyright: (c) 2010 by Stingworks
 * License: GNU GPL (see License.txt)
 *
 * File History:
 * 	18 JUL 2010 - modified from Chritian's hid-data
 * 	18 AUG 2010 - added bi-directional support using pthreads
 * 	9  SEP 2010 - ported to hidpack
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <inttypes.h>
#include "hidpack.h"
#include "pthread.h"

void *recvLoop(void * arg)
{
    uint8_t buffer[HIDPACK_PACKLEN];
	struct timeval tim;
	double timestamp;
	int err;
    long serial;

    serial = (long) arg;

	while (1) {
		if ((err = hidpack_get(serial, buffer)) != 0) {
			//fprintf(stderr, "error reading data: %s\n",
			//	usbErrorMessage(err));
		} else {
			//hexdump(buffer + 1, sizeof(buffer) - 1);
			gettimeofday(&tim, 0);
			timestamp = tim.tv_sec + (tim.tv_usec / 1000000.0);
			fprintf(stdout, "%f ", timestamp);
			fprintf(stdout, "%03d %03d %03d %03d %03d %03d %03d %03d\n",
                buffer[0],
				buffer[1],
				buffer[2],
				buffer[3],
				buffer[4], 
				buffer[5], 
				buffer[6], 
				buffer[7]);
			fflush(stdout);
		}
	}
}

void *sendLoop(void * arg)
{
	uint8_t buffer[HIDPACK_PACKLEN];	/* room for dummy report ID */
	int d[HIDPACK_PACKLEN];
	int i, pos;
	int err;
    long serial;

    serial = (long) arg;

	while (1) {
		//blocking read from stdin:
		fscanf(stdin,"%d%d%d%d%d%d%d%d",
				&d[0],
				&d[1],
				&d[2],
				&d[3], &d[4], &d[5], &d[6],
				&d[7]);
		for (i=0;i<8;i++)
			buffer[i] = d[i]; //cast

		if ((err = hidpack_put(serial, buffer)) != 0) {}
	}
}


int main(int argc, char **argv)
{
	pthread_t sendThread, recvThread;
    long serial;
    int n;
    int i;

    hidpack_init();

    if (argc == 1) {
        fprintf(stderr,"No serial specified.\n Searching for devices...\n");
        n = hidpack_find_devices();
        if (n == 0) {
            fprintf(stderr,"Found no Freefall devices... exiting.\n");
            return 0;
        } else {
            fprintf(stderr,"Found %d Freefall devices: \n",n);
            for (i = 0; i < n; i ++ ) {
                fprintf(stderr,"    SerNo: %d\n", hidpack_get_serial(i));
            }
        }
        fprintf(stderr,"Connecting to first device found...\n");
        serial = hidpack_get_serial(0);
    } else if (argc == 2) {
        serial = atoi(argv[1]);
        fprintf(stderr,"Connecting to Freefall device serial number %d ...\n",
                serial);
    } else {
        fprintf(stderr,"error: Too many arguements given. Exiting.\n");
    }


    if (hidpack_connect(serial) != 0) {
        fprintf(stderr,"error: Unable to connect to device.\n");
        return -1;
    }

	pthread_create(&sendThread, NULL, sendLoop, (void*) serial);
	pthread_create(&recvThread, NULL, recvLoop, (void*) serial);

	while (1) sleep(1);  // let send/recv threads run

    hidpack_shutdown;
	return 0;
}

/* ------------------------------------------------------------------------- */
