/* Name: hiddata.c
 * Author: Stingworks
 * Original Author: Christian Starkjohann
 * Creation Date: 2008-04-11
 * Tabsize: 4
 * Copyright: (c) 2010 Stingworks
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2
 */

#include <stdio.h>
#include <string.h>
#include <usb.h>
#include "hiddata.h"

#define usbDevice   usb_dev_handle  /* use libusb's device structure */

/* ------------------------------------------------------------------------- */

#define USBRQ_HID_GET_REPORT    0x01
#define USBRQ_HID_SET_REPORT    0x09

#define USB_HID_REPORT_TYPE_FEATURE 3


static int  usesReportIDs;

/* ------------------------------------------------------------------------- */

static int usbhidGetStringAscii(usb_dev_handle *dev, int index, char *buf, 
        int buflen)
{
char    buffer[256];
int     rval, i;
    /* use libusb version if it works */
    if((rval = usb_get_string_simple(dev, index, buf, buflen)) >= 0) 
        return rval;
    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, 
                    (USB_DT_STRING << 8) + index, 0x0409, buffer, 
                    sizeof(buffer), 5000)) < 0)
        return rval;
    if(buffer[1] != USB_DT_STRING){
        *buf = 0;
        return 0;
    }
    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];
    rval /= 2;
    /* lossy conversion to ISO Latin1: */
    for(i=1;i<rval;i++){
        if(i > buflen)              /* destination buffer overflow */
            break;
        buf[i-1] = buffer[2 * i];
        if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
            buf[i-1] = '?';
    }
    buf[i-1] = 0;
    return i-1;
}

#define USBHID_MAX_DEVICES 100
static long serials[USBHID_MAX_DEVICES];
static int serials_index;

/* return the next serial discovered.
 * if 0 is returned, that means all serials
 * have already been reported.
 */
/*int usbhidNextSerial() {
    if (serials_index < USBHID_MAX_DEVICES)
        serials_index++;
    return serials[serials_index];
}*/

long usbhidGetSerial(int i){
    return serials[i];
}

int usbhidEnumerateSerials(usbDevice_t **device, int vendor, char *vendorName, 
        int product, char *productName, int _usesReportIDs)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USBOPEN_ERR_NOTFOUND;
static int          didUsbInit = 0;
long dev_serial;

    int n;

    for (serials_index=0; serials_index<USBHID_MAX_DEVICES; serials_index++){
        serials[serials_index] = 0;
    }



    serials_index = 0;

    if(!didUsbInit){
        usb_init();
        didUsbInit = 1;
    }
    usb_find_busses();
    usb_find_devices();
    for(bus=usb_get_busses(); bus; bus=bus->next){
        for(dev=bus->devices; dev; dev=dev->next){
            if(dev->descriptor.idVendor == vendor && 
                    dev->descriptor.idProduct == product){
                char    string[256];
                int     len;
                /* we need to open the device in order to query strings */
                handle = usb_open(dev); 
                if(!handle){
                    errorCode = USBOPEN_ERR_ACCESS;
                    fprintf(stderr, "Warning: cannot open USB device: %s\n", 
                            usb_strerror());
                    continue;
                }
                /* name does not matter */
                if(vendorName == NULL && productName == NULL){  
                    break;
                }
                /* now check whether the names match: */
                len = usbhidGetStringAscii(handle, 
                        dev->descriptor.iManufacturer, string, sizeof(string));
                if(len < 0){
                    errorCode = USBOPEN_ERR_IO;
                    fprintf(stderr, 
                         "Warning: cannot query manufacturer for device: %s\n",
                         usb_strerror());
                }else{
                    errorCode = USBOPEN_ERR_NOTFOUND;
                    if(strcmp(string, vendorName) == 0){
                        len = usbhidGetStringAscii(handle, 
                                dev->descriptor.iProduct, string, 
                                sizeof(string));
                        if(len < 0){
                            errorCode = USBOPEN_ERR_IO;
                            fprintf(stderr, 
                              "Warning: cannot query product for device: %s\n",
                              usb_strerror());
                        }else{
                            errorCode = USBOPEN_ERR_NOTFOUND;
                            if(strcmp(string, productName) == 0) {
                                // check serial number
                                len = usbhidGetStringAscii(handle, 
                                            dev->descriptor.iSerialNumber, 
                                            string, sizeof(string));
                                if (len<0) {
                                    errorCode=USBOPEN_ERR_IO;
                                    fprintf(stderr, 
                               "Warning: cannot query serial for device: %s\n",
                               usb_strerror());
                                } else { 
                                    if (sscanf(string,"%ld",&dev_serial) != 1) {
                                        fprintf(stderr, 
                  "Warning: Freefall has non-number serial string: \"%s\"\n",
                  string);
                                        //override serial number
                                        string[0]='1';
                                        string[1]=NULL;
                                    }
                                    // found a device, record serial, then
                                    //  continue searching.
                                    errorCode = USBOPEN_ERR_NOTFOUND;
                                    sscanf(string,"%ld",&dev_serial);
                                    serials[serials_index++] = dev_serial;
                                }
                            }
                        }
                    }
                }
                usb_close(handle);
                handle = NULL;
            }
        }
        if(handle)
            break;
    }
    if(handle != NULL){
        errorCode = 0;
        *device = (void *)handle;
        usesReportIDs = _usesReportIDs;
    }
    n = serials_index;
    serials_index = 0;
    return n;
}


int usbhidOpenDevice(usbDevice_t **device, int vendor, char *vendorName, 
        int product, char *productName, long serial, int _usesReportIDs)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USBOPEN_ERR_NOTFOUND;
static int          didUsbInit = 0;

    if(!didUsbInit){
        usb_init();
        didUsbInit = 1;
    }
    usb_find_busses();
    usb_find_devices();
    for(bus=usb_get_busses(); bus; bus=bus->next){
        for(dev=bus->devices; dev; dev=dev->next){
            if(dev->descriptor.idVendor == vendor && 
                    dev->descriptor.idProduct == product){
                char    string[256];
                int     len;
                /* we need to open the device in order to query strings */
                handle = usb_open(dev); 
                if(!handle){
                    errorCode = USBOPEN_ERR_ACCESS;
                    fprintf(stderr, "Warning: cannot open USB device: %s\n", 
                            usb_strerror());
                    continue;
                }
                /* name does not matter */
                if(vendorName == NULL && productName == NULL){  
                    break;
                }
                /* now check whether the names match: */
                len = usbhidGetStringAscii(handle, 
                        dev->descriptor.iManufacturer, string, sizeof(string));
                if(len < 0){
                    errorCode = USBOPEN_ERR_IO;
                    fprintf(stderr, 
                         "Warning: cannot query manufacturer for device: %s\n",
                         usb_strerror());
                }else{
                    errorCode = USBOPEN_ERR_NOTFOUND;
                    if(strcmp(string, vendorName) == 0){
                        len = usbhidGetStringAscii(handle, 
                                dev->descriptor.iProduct, string, 
                                sizeof(string));
                        if(len < 0){
                            errorCode = USBOPEN_ERR_IO;
                            fprintf(stderr, 
                              "Warning: cannot query product for device: %s\n",
                              usb_strerror());
                        }else{
                            errorCode = USBOPEN_ERR_NOTFOUND;
                            if(strcmp(string, productName) == 0) {
                                // check serial number
                                if (serial == 0) {
                                    break; // don't care
                                } else {
                                    len = usbhidGetStringAscii(handle, 
                                            dev->descriptor.iSerialNumber, 
                                            string, sizeof(string));
                                    if (len <0) {
                                        errorCode=USBOPEN_ERR_IO;
                                        fprintf(stderr, 
                               "Warning: cannot query serial for device: %s\n",
                               usb_strerror());
                                    } else {
                                        errorCode = USBOPEN_ERR_NOTFOUND;
                                        long dev_serial;
                                        sscanf(string,"%ld",&dev_serial);
                                        if (dev_serial == serial) {
                                            break;
                                        } else {
                                            usb_close(handle); handle=NULL;
                                            continue;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                usb_close(handle);
                handle = NULL;
            }
        }
        if(handle)
            break;
    }
    if(handle != NULL){
        errorCode = 0;
        *device = (void *)handle;
        usesReportIDs = _usesReportIDs;
    }
    return errorCode;
}

/* ------------------------------------------------------------------------- */

void    usbhidCloseDevice(usbDevice_t *device)
{
    if(device != NULL)
        usb_close((void *)device);
}

/* ------------------------------------------------------------------------- */

int usbhidSetReport(usbDevice_t *device, char *buffer, int len)
{
int bytesSent;

    if(!usesReportIDs){
        buffer++;   /* skip dummy report ID */
        len--;
    }
    bytesSent = usb_control_msg(
            (void *)device, 
            USB_TYPE_CLASS | USB_RECIP_DEVICE | USB_ENDPOINT_OUT, 
            USBRQ_HID_SET_REPORT, 
            USB_HID_REPORT_TYPE_FEATURE << 8 | (buffer[0] & 0xff), 
            0, 
            buffer, 
            len, 
            5000);

    if(bytesSent != len){
        if(bytesSent < 0)
            fprintf(stderr, "Error sending message: %s\n", usb_strerror());
        return USBOPEN_ERR_IO;
    }
    return 0;
}

/* ------------------------------------------------------------------------- */

int usbhidGetReport(usbDevice_t *device, int reportNumber, char *buffer, int *len)
{
int bytesReceived, maxLen = *len;

    if(!usesReportIDs){
        buffer++;   /* make room for dummy report ID */
        maxLen--;
    }
    bytesReceived = usb_control_msg((void *)device, USB_TYPE_CLASS | USB_RECIP_DEVICE | USB_ENDPOINT_IN, USBRQ_HID_GET_REPORT, USB_HID_REPORT_TYPE_FEATURE << 8 | reportNumber, 0, buffer, maxLen, 5000);
    if(bytesReceived < 0){
        fprintf(stderr, "Error sending message: %s\n", usb_strerror());
        return USBOPEN_ERR_IO;
    }
    *len = bytesReceived;
    if(!usesReportIDs){
        buffer[-1] = reportNumber;  /* add dummy report ID */
        (*len)++;
    }
    return 0;
}
