/* Name: hidpack.c
 * Project: hidpack
 * Author: Stingworks
 * Based on hid-data example by Christian Starkjohann
 * Creation Date: 2010-07-18
 * Copyright: (c) 2010 by Stingworks
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt)
 *
 * File History:
 * 	18 JUL 2010 - modified from Chritian's hid-data
 * 	18 AUG 2010 - added bi-directional support using pthreads
 * 	5  SEP 2010 - created "hidpack",
 * 		removed pthreads, modularized for use directly in SteamingPot
 */
/*
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <inttypes.h>
#include "hiddata.h"
#include "usbconfig.h"		
*/
#include "hidpack.h"

/* ------------------------------------------------------------------------- */

static char *usbErrorMessage(int errCode)
{
	static char buffer[80];

	switch (errCode) {
	case USBOPEN_ERR_ACCESS:
		return "Access to device denied";
	case USBOPEN_ERR_NOTFOUND:
		return "The specified device was not found";
	case USBOPEN_ERR_IO:
		return "Communication error with device";
	default:
		sprintf(buffer, "Unknown USB error %d", errCode);
		return buffer;
	}
	return NULL;		/* not reached */
}

static usbDevice_t *openDevice( long serial )
{
	usbDevice_t *dev = NULL;
	unsigned char rawVid[2] = { USB_CFG_VENDOR_ID }, rawPid[2] = {
	USB_CFG_DEVICE_ID};
	char vendorName[] = { USB_CFG_VENDOR_NAME, 0 }, productName[] = {
	USB_CFG_DEVICE_NAME, 0};
	int vid = rawVid[0] + 256 * rawVid[1];
	int pid = rawPid[0] + 256 * rawPid[1];
	int err;

	if ((err =
	     usbhidOpenDevice(&dev, vid, vendorName, pid, productName, serial, 0)) != 0)
	{
		fprintf(stderr, "error finding %s: %s\n", productName,
			usbErrorMessage(err));
		return NULL;
	}
	return dev;
}

/* ------------------------------------------------------------------------- */

struct usbDeviceWrapper_t {
	usbDevice_t *handle;
	int allocated;
	long serial;
};

static struct usbDeviceWrapper_t dev[HIDPACK_MAX_DEVICES];

static int dev_find_serial(long serial) {
	int i;
	for (i = 0; i < HIDPACK_MAX_DEVICES; i++) {
		if ((dev[i].serial == serial || serial == 0) && dev[i].allocated == 1)
			return i;
	}
	return -1;
}


static int next_free_dev() {
	int i;
	for (i = 0; i < HIDPACK_MAX_DEVICES; i++) {
		if (!dev[i].allocated)
			return i;
	}
	return -1;
}

/* ------------------------------------------------------------------------ */
/* - LIBRARY INTERFACE 							  - */
/* ------------------------------------------------------------------------ */

int hidpack_init() {
	int i;
	for (i = 0; i < HIDPACK_MAX_DEVICES; i++) {
		dev[i].handle = NULL;
		dev[i].allocated = 0;
		dev[i].serial = 0;
	}
    return 0;
}

int hidpack_find_devices() {
    usbDevice_t *dev = NULL;
	unsigned char rawVid[2] = { USB_CFG_VENDOR_ID }, rawPid[2] = {
	USB_CFG_DEVICE_ID};
	char vendorName[] = { USB_CFG_VENDOR_NAME, 0 }, productName[] = {
	USB_CFG_DEVICE_NAME, 0};
	int vid = rawVid[0] + 256 * rawVid[1];
	int pid = rawPid[0] + 256 * rawPid[1];
	int err;

    // generate internal list of serials
    return usbhidEnumerateSerials(dev, vid, vendorName, pid, productName, 0);
}

/*int hidpack_next_device() {
    return usbhidNextSerial();
}*/

long hidpack_get_serial(int i) {
    return usbhidGetSerial(i);
}



// open communication with Freefall with given serial number
//  serial = 0 means any
int hidpack_connect( long serial )
{
	int i = next_free_dev();
	dev[i].serial = serial;

	// access the HID device
	if ((dev[i].handle = openDevice( dev[i].serial )) == NULL)
		return -1;

	dev[i].allocated = 1;

	return 0;
}

int hidpack_get(long serial, uint8_t * packet) {
	uint8_t buffer[HIDPACK_PACKLEN+1];
	int err, len = sizeof(buffer);
	int i = dev_find_serial(serial);
	if (i < 0) {fprintf(stderr, "hidpack: device with serial %d not initialized!\n", serial);}
	if ((err = usbhidGetReport(dev[i].handle, 0, buffer, &len)) != 0) {
	    // suppress error message
        //fprintf(stderr, "error reading data: %s\n",
		//	usbErrorMessage(err));
	}
	memcpy(packet,buffer+1,HIDPACK_PACKLEN);
	return err;
}

int hidpack_put(long serial, const uint8_t * packet) {
	int err;
	uint8_t buffer[HIDPACK_PACKLEN+1];
	int i = dev_find_serial(serial);
	if (i < 0) {fprintf(stderr, "hidpack: device with serial %d not initialized!\n", serial);}
	memcpy(buffer+1,packet,HIDPACK_PACKLEN);
	buffer[0] = 0;
	if ((err = usbhidSetReport(dev[i].handle, buffer, sizeof(buffer))) != 0)	
		fprintf(stderr, "error writing data: %s\n",
			usbErrorMessage(err));
	return err;
}

int hidpack_disconnect(long serial)
{
	int i = dev_find_serial(serial);
	if (i < 0) {
		fprintf(stderr, "hidpack: device with serial %d not initialized!\n", serial);
		return -1;
	}
	usbhidCloseDevice(dev[i].handle);
	dev[i].allocated = 0;
	return 0;
}

int hidpack_shutdown()
{
	int i;
	// No! Shut them all down!
	for (i = 0; i < HIDPACK_MAX_DEVICES; i++) {
		if (dev[i].allocated) {
			hidpack_disconnect(dev[i].serial);
		}
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
