
/*****************************************************
SteamingPot draws streaming plots
 (A live plotting program for the Stingworks Freefall)
(c) Stingworks 2010
This program may be distributed under the terms of the
 GNU General Public License version 2 or later.
Compiles best on Linux (but may compile on other systems)
*****************************************************/ 

#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <time.h>
#include <SDL.h>
#include "SDL_ttf.h"
#include "SDL_gfxPrimitives.h"
#include "hidpack.h"
#include "swmath.h"
#include "swgfx.h"

//define size of screen
#define XMAX 1000
#define YMAX 700 


int main(int argc, char* argv[])
{
  SDL_Surface *screen;
  SDL_Event event;

  int kspace = false, kup = false, kdown = false, kleft = false, kright = false;
  int my, mx;

  srand(1);  /* Fetch a set of fresh random numbers */ 
  
  /* Initialize SDL, exit if there is an error. */
  if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Could not initialize SDL: %s\n", 
      SDL_GetError());
    return -1;
  }

  TTF_Init();
  
  /* When the program is through executing, call SDL_Quit */
  atexit(SDL_Quit);
  atexit((void*) hidpack_shutdown);
  
  /* Grab a surface on the screen */
  screen = SDL_SetVideoMode(XMAX, YMAX, 32, SDL_SWSURFACE|SDL_ANYFORMAT);
  if( !screen ) {
    fprintf(stderr, "Couldn't create a surface: %s\n",
      SDL_GetError());
    return -1;
  }

  SDL_WM_SetCaption("SteamingPot - Real-Time Plotting of Freefall Data", 
          "steamingpot.png" );

  double x = 0;
  double y = 0;
  double z = 0;
  double rawx,rawy,rawz;
  int xoff,yoff,zoff;
  int ox = 0;
  int oy = 0;
  int oz = 0;
  float t = 0;
  int sample = 0;
  float ot = 0;
  const float smooth = .85;
  float cds, pman, btemp,pman2;
  float vbatt=0,vusb=0;
  uint8_t d[8];
  float theta, psi, stheta = 0, spsi = 0;
  int b, b2;
  long ff_serial = 0, ff_n = 0;
  int err;

#define HIST_SIZE XMAX/2
  double hist[HIST_SIZE][3];
  int i,j;
  for (i=0;i<HIST_SIZE;i++) 
	  for (j=0;j<3;j++)
		  hist[i][j]=0;

  // initialize accelerometer filters
  SWParticle * xparticle = malloc(sizeof(SWParticle));
  SWParticle * yparticle = malloc(sizeof(SWParticle));
  SWParticle * zparticle = malloc(sizeof(SWParticle));
  swparticle_init(xparticle);
  swparticle_init(yparticle);
  swparticle_init(zparticle);


  // connect to device
  
  hidpack_init();

    if (argc == 1) {
        fprintf(stderr,"No serial specified.\n Searching for devices...\n");
        ff_n = hidpack_find_devices();
        if (ff_n == 0) {
            fprintf(stderr,"Found no Freefall devices... exiting.\n");
            return 0;
        } else {
            fprintf(stderr,"Found %d Freefall devices: \n",ff_n);
            for (i = 0; i < ff_n; i ++ ) {
                fprintf(stderr,"    SerNo: %d\n", hidpack_get_serial(i));
            }
        }
        fprintf(stderr,"Connecting to first device found...\n");
        ff_serial = hidpack_get_serial(0);
    } else if (argc == 2) {
        ff_serial = atoi(argv[1]);
        fprintf(stderr,"Connecting to Freefall device serial number %d ...\n",
                ff_serial);
    } else {
        fprintf(stderr,"error: Too many arguements given. Exiting.\n");
    }

    if (hidpack_connect(ff_serial) != 0) {
        fprintf(stderr,"error: Unable to connect to device.\n");
        return -1;
    }


     /* START OF MAIN PROGRAM LOOP */
  do {

	  // do interesting shit here:
	  ot = t; ox = x; oy = y; oz = z;

	  if (err == 0) err = hidpack_get(ff_serial,d);
      if (err != 0) {
          if(system("avrdude -c usbasp -p m168 -qq") == 0) {
	        drawTextColor(screen,"RECONNECTING...",20,100,20,GREY,BLACK);
	        SDL_Flip(screen); 
            sleep(1);
          } else {
	        drawTextColor(screen,"DISCONNECTED",20,100,20,GREY,BLACK);
	        SDL_Flip(screen); 
          }
          sleep(1);
          hidpack_init();
          err = hidpack_connect(ff_serial);
      }

      if (err == 0) {

	    sample++;

        // unpack raw acc data
	    rawx = ((int8_t *) d)[1];
	    rawy = ((int8_t *) d)[2];
	    rawz = ((int8_t *) d)[3];

        // apply particle filter
        x = swparticle_update(xparticle,rawx);
        y = swparticle_update(yparticle,rawy);
        z = swparticle_update(zparticle,rawz);

    	cds = unpack_float(d[4],0,100);
    	pman=d[5];
        pman2=d[6];
        btemp =  unpack_float(d[7],10,50);

	    hist[sample % HIST_SIZE][0] = x;
	    hist[sample % HIST_SIZE][1] = y;
	    hist[sample % HIST_SIZE][2] = z;

        /*x = rawx;
        y = rawy;
        z = rawz;*/

	    // normalize acceleration vector
	    double magn = sqrt(x*x + y*y + z*z);
	    if (magn != 0) {
	    	x /= magn;
	  	    y /= magn;
	  	    z /= magn;
	    } else {
		    x = y = z = 0;
	    }


	    // compute pitch and roll
	    theta = asin(-x);

	    if (z > 0) {
		  psi = atan(y/z);
	    } else if (z < 0) {
		  psi = PI + atan(y/z);
	    } else if (y > 0) {
		  psi = PI/2;
	    } else {
		  psi = 3*PI/2;
	    }
	    if (psi > PI) psi -= 2*PI;
      }

	  stheta = weighted_angle_average(stheta, theta, smooth);
	  spsi = weighted_angle_average(spsi , psi, smooth);

      vusb = weighted_average(vusb, unpack_float(pman,2,6), smooth);
      vbatt = weighted_average(vbatt, unpack_float(pman2,2,6), smooth);



	  // do graphics only every few samples
	  if (sample % 8 == 0) 
	  {
		  SDL_Flip(screen); 
	      SDL_FillRect( screen, NULL, SDL_MapRGB(screen->format,0,0,0) );
          int pos;
	  for ( pos = 1; pos < HIST_SIZE; pos++) {
		// draw +-1g lines
		for ( i = -1; i <= 1; i++) {
			if (i == 0) continue;
			b = (sin(pos/3.)+1)*30;
			setpixel(screen,pos,130+i*32,0,b,0);
	  		setpixel(screen,pos,330+i*32,0,b,0);
	  		setpixel(screen,pos,530+i*32,0,b,0);
		}

		b = pos*1024/HIST_SIZE;
		if (b > 180) b2 = (b - 180) * 2; else b2 = 0;
		i = (sample - (HIST_SIZE - pos)) % HIST_SIZE;


		if (i < 1) i+= HIST_SIZE-1;
	  	setpixel(screen,pos,130,10,100,10);
	  	setpixel(screen,pos,330,10,100,10);
	  	setpixel(screen,pos,530,10,100,10);
	  	/*setpixel(screen,pos,130-hist[i][0],b,b,b2);
	  	setpixel(screen,pos,330-hist[i][1],b,b,b2);
	  	setpixel(screen,pos,530-hist[i][2],b,b,b2);*/
        aalineColor(screen,
                pos-1, 130-hist[i-1][0], 
                pos,   130-hist[i][0],
                0xFFFFFF00+b);
        aalineColor(screen,
                pos-1, 330-hist[i-1][1], 
                pos,   330-hist[i][1],
                0xFFFFFF00+b);
        aalineColor(screen,
                pos-1, 530-hist[i-1][2], 
                pos,   530-hist[i][2],
                0xFFFFFF00+b);

	  }

#define GLEFT   XMAX*5/8
#define GRIGHT  XMAX*7/8
#define GTOP    YMAX/6
#define GMID    YMAX/2
#define GLOW    YMAX*5/6
#define GSIZE   XMAX/10

      drawGuage(screen, GLEFT, GTOP, GSIZE,
		"pitch", 
		-270, 90, 90, 10,
		1000,
		stheta*180./PI);

	  drawGuage(screen, GRIGHT, GTOP  , GSIZE,
		"roll", 
		-180, 180, 90, 10,
		1000,
		spsi*180./PI);

        drawGuage(screen, GLEFT   ,GMID  ,GSIZE  ,
                "CDS (%)",
                0,99.9,10,5,
                500,
                cds);

        drawGuage(screen, GRIGHT  ,GMID  ,GSIZE  ,
                "BTEMP (C)",
                0,79.9,20,5,
                90,
                btemp);

        drawGuage(screen, GLEFT   ,GLOW    ,GSIZE,
              "Vbatt (V)",
              0,6.99,1,.25,
              5.5,
              vbatt);

        drawGuage(screen, GRIGHT ,GLOW    ,GSIZE,
              "Vext (V)",
              0,6.99,1,.25,
              5.5,
              vusb);

		drawTextColor(screen,"x-axis",12,20,130+15,GREEN,BLACK);
		drawTextColor(screen,"y-axis",12,20,330+15,GREEN,BLACK);
		drawTextColor(screen,"z-axis",12,20,530+15,GREEN,BLACK);
        /*drawGuage(screen, GRIGHT  ,GLOW    ,GSIZE,
              "Ibatt (mA)",
              0,699.9,100,10,
              500,
              floor((vusb-vbatt)/1.*1000));*/

      }





      

	  
	  //determine frame rate

	  SDL_PumpEvents();
	  SDL_GetMouseState(&mx, &my);

  while(SDL_PollEvent(&event)) {
  switch( event.type ) 
  {
  case SDL_QUIT:
    printf("Got quit event!\n");
    return 0;
    break;
  case SDL_KEYDOWN:
    switch( event.key.keysym.sym ) {
      case SDLK_ESCAPE:
	printf("Got quit key!\n");
	return 0;
	break;
      case SDLK_UP:
	kup = true;
	break;
      case SDLK_DOWN:
	kdown = true;
        break;
      case SDLK_LEFT:
	kleft = true;
	break;
      case SDLK_RIGHT:
	kright = true;
	break;
      case SDLK_SPACE:
	kspace = true;
	break;
      case SDLK_TAB:
	break;
      default:
	break;
    }
    break;
  case SDL_KEYUP:
    switch( event.key.keysym.sym ) {
      case SDLK_UP:
	kup = false;
	break;
      case SDLK_DOWN:
	kdown = false;
        break;
      case SDLK_LEFT:
	kleft = false;
	break;
      case SDLK_RIGHT:
	kright = false;
	break;
      case SDLK_SPACE:
	kspace = false;
	break;
      case SDLK_TAB:
	break;
      default:
	break;
    }
    break;
  case SDL_MOUSEBUTTONDOWN:
    break;
  case SDL_MOUSEBUTTONUP:
    break;
  default:
	break;
  }
  }

  } while (1);
  
  printf("should never reach here\n");
  
  return 0;
}


