/* 
 * Usart-Stream
 *
 * Example code to read data from Freefall via USART.
 *
 */

#include<stdio.h>
#include<inttypes.h>
#include<sys/time.h>
#include<math.h>

#include"packetizer.h"
#include"freefall_protocol.h"


#define SAVE_PACKET_STREAM	stdout


void save_acc_packet(uint8_t * data)
// display data from incomming packet in human readable form
{
	int8_t ax = ((int8_t *) data)[FREEFALL_AX];
	int8_t ay = ((int8_t *) data)[FREEFALL_AY];
	int8_t az = ((int8_t *) data)[FREEFALL_AZ];
	uint8_t cds = data[FREEFALL_CDS];

	struct timeval tim;
	gettimeofday(&tim, 0);
	double timestamp = tim.tv_sec+(tim.tv_usec/1000000.0);

	fprintf(SAVE_PACKET_STREAM,"%f %d %d %d %d %d %d %d %d\n",
		       	timestamp,
			data[0],
			ax,
			ay,
			az,
			cds,
			data[FREEFALL_PMON],
			data[FREEFALL_BTEMP],
			data[FREEFALL_RESERVED]
			);

	fflush(stdout);
	
}

void save_packet(uint8_t * data)
// write packet data to stdout
{

	struct timeval tim;
	gettimeofday(&tim, 0);
	double timestamp = tim.tv_sec+(tim.tv_usec/1000000.0);

	fprintf(SAVE_PACKET_STREAM,"%f,%d,%d,%d,%d,%d,%d,%d,%d\n",
			timestamp,
			data[0],
			data[FREEFALL_AX],
			data[FREEFALL_AY],
			data[FREEFALL_AZ],
			data[FREEFALL_CDS],
			data[FREEFALL_PMON],
			data[FREEFALL_BTEMP],
			data[FREEFALL_RESERVED]);
}
		


void main() {

	uint8_t length;
	uint8_t buffer[FREEFALL_PACKET_LENGTH + 1];
	int failed_packet_count = 0;

	packetizer_init(getchar,0);

	while (1) {
		length = packetizer_read(buffer,FREEFALL_PACKET_LENGTH);
		if (length > 127) { // check for error signal
			fprintf(stderr,"Error reading last %d packets.\n",\
					failed_packet_count);
			failed_packet_count++;
			continue;
		} else {
			failed_packet_count=0;
		}
		save_acc_packet(buffer);
	}

	return;
}

