/*
 * adc.h - Analog to Digital Converter driver
 *
 *   Copyright (c) 2002 Pascal Stang
 *   Copyright (c) 2010 Stingworks
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *
 *
 * File History:
 *  
 *   10 FEB 2010:  ported from Procyon
 *
 */

#define ADC_PRESCALE_DIV2		0x00	
#define ADC_PRESCALE_DIV4		0x02
#define ADC_PRESCALE_DIV8		0x03
#define ADC_PRESCALE_DIV16		0x04
#define ADC_PRESCALE_DIV32		0x05
#define ADC_PRESCALE_DIV64		0x06
#define ADC_PRESCALE_DIV128		0x07
#define ADC_PRESCALE_MASK		0x07

#define ADC_REFERENCE_AREF     		0x00
#define ADC_REFERENCE_AVCC     		0x01
#define ADC_REFERENCE_INTERNAL 		0x03 
#define ADC_REFERENCE_DEFAULT     	ADC_REFERENCE_AVCC
#define ADC_REFERENCE_MASK		0xC0

#define ADC_MUX_MASK			0x1F


void adc_set_prescale(unsigned char prescale) {
	ADCSRA = ADCSRA & ~ADC_PRESCALE_MASK | prescale;
}

void adc_set_reference(unsigned char ref) {
	ADMUX = ADMUX & ~ADC_REFERENCE_MASK | ref << 6;
}

void adc_init(void) {

	ADCSRA |= 1<<ADEN;  	// turn on ADC power

	ADCSRA &= ~(1<<ADATE);	// single sample convert mode (?)

	adc_set_prescale(ADC_PRESCALE_DIV64); // set default prescaler

	adc_set_reference(ADC_REFERENCE_DEFAULT); // set default reference

	//ADMUX &= ~(1<<ADLAR); // set to right-adjusted result (?)
	ADMUX |= (1<<ADLAR); // set to left-adjusted result

	ADCSRA &= ~(1<<ADIE); // disable ADC interrupts

}

void adc_off(void) {
	// turn off ADC and disable interrupts
	ADCSRA &= ~( 1<<ADIE | 1<<ADEN ); 
}

void adc_set_channel(unsigned char ch) {
	ADMUX = ADMUX & ~ADC_MUX_MASK | ch & ADC_MUX_MASK;
}

void adc_start_conversion(void) {
	// "clear" conversion complete flag
	//  and begin conversion
	ADCSRA |= 1<<ADIF | 1<<ADSC;
}

unsigned char adc_conversion_complete(void) {
	return (ADCSRA & 1<<ADSC) == 0;
}

unsigned short adc_read_10bit(unsigned char ch) {
	unsigned short retval;
	adc_set_channel(ch);
	adc_start_conversion();
	while(!adc_conversion_complete());
	retval = ADCL / 64; // must read ADCL before ADCH!
	retval += ADCH * 4;
	return retval;
}

unsigned char adc_read(unsigned char ch) {
	// downconvert a 10bit reading
	//return adc_read_10bit(ch) >> 2; //not functioning
	
	adc_set_channel(ch);
	adc_start_conversion();
	while(!adc_conversion_complete());
	return ADCH;
}

